% Dynamics function for a set of systems
function [StatesDot] = dynamics(t, States_all, param)
      
    % Extracting parameters from param cell array
    n                  = param{1};     % Number of states for each system
    m                  = param{2};     % Number of inputs for each system
    omega_l            = param{3};     % Low pass frequency
    omega_h            = param{4};     % High pass frequency
    Phi                = param{5};     % Matrix Phi for each system
    k                  = param{6};     % Coefficients for system dynamics
    f                  = param{7};     % Function handles for system dynamics
    J                  = param{8};     % Function handles for output
    sistem_state_size  = param{9};     % Size of state for each system
    N                  = param{10};    % Number of systems
    
    % Calculating the indices of states for each system
    sistem_index        = [0, cumsum(sistem_state_size(2, :))];
    sistem_state_index  = sistem_index(1:end - 1);
    
    % Extracting the states for each system
    x_all = [];
    for j = 1:N
       x_all = [x_all; States_all(sistem_state_index(j) + 1: sistem_state_index(j) + n(j))]; 
    end
    
    % Initialize index and state derivatives
    i = cell(1, N);
    StatesDot = cell(N, 1);
    
    % Loop over each system
    for j = 1 : N
        States = States_all(sistem_index(j) + 1 : sistem_index(j + 1));

        % Define indices for different parts of the state vector
        i{j} = zeros(1, 6);

        i{j}(1) = 0;
        i{j}(2) = i{j}(1) + n(j);    
        i{j}(3) = i{j}(2) + m(j);
        i{j}(4) = i{j}(3) + m(j);
        i{j}(5) = i{j}(4) + 1;    
        i{j}(6) = i{j}(5) + 2 * m(j);

        % Extract different parts of the state vector
        x   = States(i{j}(1) + 1 : i{j}(2));     % State of system
        u   = States(i{j}(2) + 1 : i{j}(3));     % Input
        ksi = States(i{j}(3) + 1 : i{j}(4));     % Auxiliary state variable
        ni  = States(i{j}(4) + 1 : i{j}(5));     % Auxiliary state variable
        d   = States(i{j}(5) + 1 : i{j}(6));     % Control input perturbation

        % Calculate output of the system
        y = J{j}(x_all);

        % Calculate state derivatives
        StatesDot{j} = zeros(size(States));

        StatesDot{j}(i{j}(1) + 1 : i{j}(2)) = f{j}(x, u + d(1:2:end));
        StatesDot{j}(i{j}(2) + 1 : i{j}(3)) = -k(j) * ksi;
        StatesDot{j}(i{j}(3) + 1 : i{j}(4)) = -omega_l(j) * ksi + omega_l(j) * (y - ni) .* d(1:2:end);
        StatesDot{j}(i{j}(4) + 1 : i{j}(5)) = -omega_l(j) * ni + omega_h(j) * y;
        StatesDot{j}(i{j}(5) + 1 : i{j}(6)) = Phi{j} * d;
    end
    % Convert state derivatives cell array to a matrix
    StatesDot = cell2mat(StatesDot);
end
