function [StatesDot] = single_step(t, States, multiple, param)
    % Function to compute the derivative of the system states for a single step
    
    % Extract parameters from the input cell array 'param'
    m         = param{1};    % Dimension of control variables
    Phi       = param{2};    % Matrix of control variable coefficients
    Sigma     = param{3};    % Coefficient matrix for sigma calculation
    kt        = param{4};    % Coefficient for sigma calculation
    K         = param{5};    % Coefficient for y_kappa calculation
    step      = param{6};    % Step size for projection
    Omega_box = param{9};    % Bounds for the control variables
    epsilon   = param{10};   % Step size for the dynamics
    sys_dym   = param{11};   % Function handle for the system dynamics
    sys_par   = param{12};   % System parameters
    n         = param{13};   % Dimension of state variables
    noise     = param{14};   % Noise vector
    
    i(1) = n;
    i(2) = i(1) + 1;
    i(3) = i(2) + m + 1;
    i(4) = i(3) + 1;
    i(5) = i(4) + (m + 1).^2;
    i(6) = i(5) + m + 1;
    i(7) = i(6) + 2 * m;
    i(8) = i(7) + m;
    i(9) = i(8) + 1;
    
    x          = States(       1 : i(1));    % State variables
    y_kapa     = States(i(1) + 1 : i(2));    % y kappa
    c          = States(i(2) + 1 : i(3));    % c
    ni_kapa    = States(i(3) + 1 : i(4));    % ni kappa
    sigma      = States(i(4) + 1 : i(5));    % sigma
    theta_kapa = States(i(5) + 1 : i(6));    % theta kappa
    d          = States(i(6) + 1 : i(7));    % d
    u_kapa     = States(i(7) + 1 : i(8));    % u kappa
    y          = States(i(8) + 1 : i(9)) + noise(1, floor(t) + 1);   % y
    
    e           = y - y_kapa;                               % Error calculation
    sigma       = reshape(sigma, m + 1, m + 1);             % Reshape sigma matrix
    sigmaDot    = c * c' - kt * sigma + Sigma * eye(m + 1); % Derivative of sigma matrix
    StatesDot   = zeros(size(States));                      % Initialize the derivative of states vector
    
    StatesDot(       1 : i(1))  = sys_dym(t, x, u_kapa, sys_par);   % Derivative of state variables
    StatesDot(i(3) + 1 : i(4))  = epsilon * -K * ni_kapa;           % Derivative of ni kappa
    StatesDot(i(4) + 1 : i(5))  = epsilon * sigmaDot(:);            % Derivative of sigma
    StatesDot(i(5) + 1 : i(6))  = epsilon * sigma^(-1) * (c * (e - ni_kapa) - Sigma * theta_kapa); % Derivative of theta kappa
    StatesDot(i(6) + 1 : i(7))  = epsilon * Phi * d;                % Derivative of d
    udot                        = -u_kapa + proj_box(u_kapa - step * (theta_kapa(2:end) + multiple) + d(1:2:end), Omega_box);
    StatesDot(i(7) + 1 : i(8))  = epsilon * udot;                   % Derivative of u kappa
    phi                         = [1; udot];                        % Auxiliary variable
    StatesDot(i(2) + 1 : i(3))  = epsilon * (-K*c + phi);           % Derivative of c
    StatesDot(i(1) + 1 : i(2))  = epsilon * (phi' * theta_kapa + K * e + c'*StatesDot(i(5) + 1 : i(6))); % Derivative of y kappa
    StatesDot(i(8) + 1 : i(9))  = 0;                                % Derivative of unused variable
    
end
