function [States] = dynamics_single_step(t, States, param, N)
    % DYNAMICS_SINGLE_STEP is a function to compute a single step in the dynamical system.
    %
    % Inputs:
    %   - t: current time
    %   - States: a vector containing the current states of the system
    %   - param: a cell array containing parameters used in the system
    %   - N: total number of wind turbines
    %
    % Output:
    %   - States: a vector containing the updated states of the system

    % Extract parameters
    h       = param{N + 1 + 1};  % h function for mapping states
    u_index = param{N + 1 + 2};  % indices for control variables
    y_index = param{N + 1 + 3};  % indices for output variables
    indices = param{N + 1 + 4};  % indices for partitioning the States vector
    x_index = param{N + 1 + 5};  % indices for state variables

    % Additional parameters for the system
    A_con   = param{N + 1}{1};   % matrix A for constraints
    b_con   = param{N + 1}{2};   % vector b for constraints
    ni      = param{N + 1}{3};   % ni parameter
    m       = param{N + 1}{4};   % m parameter
    q       = param{N + 1}{5};   % q parameter

    % Extract current states
    x       = States(x_index);   % state variables
    u       = States(u_index);   % control variables
    lambda  = States(indices(N + 1) + 1 : end);  % Lagrange multipliers

    % Compute multiplier
    multipl = A_con' * lambda;
    multipl = reshape(multipl, m, []);

    % Compute h(x)
    States(y_index) = h(x);

    % Single-step update for each subsystem
    for i = 1:N
        States(indices(i) + 1 : indices(i + 1)) = single_step(t, States(indices(i) + 1 : indices(i + 1)), multipl(:, i), param{i});
    end

    % Update Lagrange multipliers
    States(indices(N + 1) + 1 : end) = - lambda + proj_plus(lambda + ni * (A_con * (u + 2 * States(u_index)) - b_con));  
end
