function [States] = single_step(t, States, multiple, param)
    % SINGLE_STEP is a function to compute a single step in the dynamical system.
    %
    % Inputs:
    %   - t: current time
    %   - States: a vector containing the current states of the system
    %   - multiple: a multiplier used in the system
    %   - param: a cell array containing parameters used in the system
    %
    % Output:
    %   - States: a vector containing the updated states of the system

    % Extract parameters
    m       = param{1};         % m parameter
    Phi     = param{2};         % Phi parameter
    Sigma   = param{3};         % Sigma parameter
    kt      = param{4};         % kt parameter
    K       = param{5};         % K parameter
    step    = param{6};         % step size
    Omega_box = param{9};       % Omega_box parameter
    epsilon = param{10};        % epsilon parameter
    sys_dym = param{11};        % system dynamics
    sys_par = param{12};        % system parameters
    n       = param{13};        % n parameter

    % Define indices
    i       = zeros(9,1);
    i(1)    = n;    
    i(2)    = i(1) + 1;
    i(3)    = i(2) + m + 1;    
    i(4)    = i(3) + 1;
    i(5)    = i(4) + (m + 1).^2;
    i(6)    = i(5) + m + 1;
    i(7)    = i(6) + 2 * m;
    i(8)    = i(7) + m; 
    i(9)    = i(8) + 1;

    % Extract current states
    x           = States(       1 : i(1));                 % state variables
    y_kapa      = States(i(1) + 1 : i(2));                 % y kappa variable
    c           = States(i(2) + 1 : i(3));                 % c variable
    ni_kapa     = States(i(3) + 1 : i(4));                 % ni kappa variable
    sigma       = States(i(4) + 1 : i(5));                 % sigma variable
    theta_kapa  = States(i(5) + 1 : i(6));                 % theta kappa variable
    d           = States(i(6) + 1 : i(7));                 % d variable
    u_kapa      = States(i(7) + 1 : i(8));                 % u kappa variable
    y           = States(i(8) + 1 : i(9));                 % y variable

    e           = y - y_kapa;                              % error between y and y kappa
    sigma       = reshape(sigma, m + 1, m + 1);            % reshape sigma
    sigmaDot    = c * c' - kt * sigma + Sigma * eye(m + 1);% rate of change of sigma

    % Update states
    States(       1 : i(1))  = sys_dym(t, x, u_kapa, sys_par);                                       % update state variables
    States(i(3) + 1 : i(4))  = epsilon * -K * ni_kapa;                                                % update ni kappa
    States(i(4) + 1 : i(5))  = epsilon * sigmaDot(:);                                                 % update sigma
    States(i(5) + 1 : i(6))  = epsilon * sigma^(-1) * (c * (e - ni_kapa) - Sigma * theta_kapa);       % update theta kappa
    States(i(6) + 1 : i(7))  = epsilon * Phi * d;                                                     % update d
    udot                     = -u_kapa + proj_box(u_kapa - step * (theta_kapa(2:end) + multiple) + d(1:2:end), Omega_box); 
    States(i(7) + 1 : i(8))  = epsilon * udot;                                                        % update u kappa
    phi                      = [1; udot];
    States(i(2) + 1 : i(3))  = epsilon * (-K*c + phi);                                                % update c
    States(i(1) + 1 : i(2))  = epsilon * (phi' * theta_kapa + K * e + c'*States(i(5) + 1 : i(6)));    % update y kappa
    States(i(8) + 1 : i(9))  = 0;                                                                     % reset y

end
