% Define the dimensions of the problem
m_i         = 2; % Number of control inputs per subsystem
N           = 4; % Number of subsystems
m           = N * m_i; % Total number of control inputs
n           = m_i; % Number of states in each subsystem

rng(13); % Set the seed for random number generation

% Constructing the cost function
a = 0.08;

% Initialize Q, which will be used to construct a block diagonal matrix for the cost function
Q_proto = -eye(m_i) * a;
Q = repmat(Q_proto, N, N);
Q(eye(m) == 1) = (1 + (N - 1) * a/2); 

% Initialize R, the control weighting matrix
R_s = [-4, -8; -12, -3; 1, 7; 16, 8]';
R = R_s(:);

% Define indices for different elements in the state space
indices     = 1:m;
indices_all = indices;

% Define indices for each subsystem
indices_j   = reshape(indices_all, n, [])';

indices_minus_j = zeros((n) * (N - 1), N);
for j = 1:N
    temp = indices_j';
    temp(:, j) = [];
    indices_minus_j(:, j) = temp(:);
end
indices_minus_j = indices_minus_j';

indices_reg = reshape(1 : (n) * N, n, [])';

% Define the vector of cost functions, J, as a function of the state, x
J  =  @(x)[[1; 1] * (x(indices_j(1, :))' * Q(indices_reg(1, :), :) * x(indices_all) + (R(indices_reg(1, :)) - 2 * x(indices_j(1, :)))' *  R(indices_reg(1, :)) + a/2 * x(indices_minus_j(1, :))' * x(indices_minus_j(1, :))); ...
          [1; 1] * (x(indices_j(2, :))' * Q(indices_reg(2, :), :) * x(indices_all) + (R(indices_reg(2, :)) - 2 * x(indices_j(2, :)))' *  R(indices_reg(2, :)) + a/2 * x(indices_minus_j(2, :))' * x(indices_minus_j(2, :))); ...
          [1; 1] * (x(indices_j(3, :))' * Q(indices_reg(3, :), :) * x(indices_all) + (R(indices_reg(3, :)) - 2 * x(indices_j(3, :)))' *  R(indices_reg(3, :)) + a/2 * x(indices_minus_j(3, :))' * x(indices_minus_j(3, :))); ...
          [1; 1] * (x(indices_j(4, :))' * Q(indices_reg(4, :), :) * x(indices_all) + (R(indices_reg(4, :)) - 2 * x(indices_j(4, :)))' *  R(indices_reg(4, :)) + a/2 * x(indices_minus_j(4, :))' * x(indices_minus_j(4, :)))]; %...

% Calculate Nash equilibrium
Qsol = Q;
Qsol(eye(m) == 1) = Qsol(eye(m) == 1) *2;
NE = 2 * inv(Qsol) * R;
NE = reshape(NE, 2, []);

% Generate random initial phase for each oscillator
phi = [1:m]' + (rand(m, 1) - 0.5) * 0.5 ; 

% Define the rotation matrix for each oscillator
Ri  = cell(m);
for i = 1:m
    Ri{i} = [cos(phi(i)) -sin(phi(i)); sin(phi(i)) cos(phi(i))];
end
Rcal  = diag_matrices(Ri);

% Define simulation parameters
time            = 100000;
time_interval   = [0, time];
j_lim           = 50000;
T               = [0.01, 0.015, 0.02, 0.01]; 
gamma           = 0.1;
A               = 0.1;
beta            = 0.003;
tau0            = [0 0.002 0.004 0.006]';

% Initialize the state vector
x0 = rand(m * 1, 1);
States_init = [x0; tau0; zeros(m * 1, 1); repmat([1; 0], m, 1); zeros(m, 1)]; 

i = zeros(3, 1);
i(2) = i(1) + m;
i(3) = i(2) + N;
i(4) = i(3) + m;
i(5) = i(4) + 2 * m;
i(6) = i(5) + m;

global param

% Define parameters that will be used in the ODE function
param    = {};
param{1} = m;         
param{2} = i;     
param{3} = T;         
param{4} = gamma;   
param{5} = J;
param{6} = Rcal;
param{7} = m_i;
param{8} = N;
param{9} = A;
param{10} = beta;
param{11} = phi;

% Simulation horizon
TSPAN = [0 time];
JSPAN = [0 j_lim];

% rule for jumps
% rule = 1 -> priority for jumps
% rule = 2 -> priority for flows
rule = 1;

options = odeset('RelTol',1e-7,'MaxStep',.01);

% Start the simulation
tic
[t,j,x] = HyEQsolver( @f, @g, @C, @D, States_init, TSPAN, JSPAN, rule, options, 'ode45');
toc

% Select every second column of the oscilating solution since even ones are
% pi/2 different than the odd ones
mi      = x(:, i(4) + 1 : 2 : i(5));


%% x time plot with oscilations
plot_sample = 100;
last_index  = 250000;

figure % position
figure_standard
hold on
%clf

% Plot the positions of the agents in time.
% The plot function is used with a time delta to reduce the number of plotted points and thus increase plot efficiency.

% Blue represents Agent 1
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(1) + 1) + A * mi(1:plot_sample:last_index, 1), 'b', 'LineWidth', 2)
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(1) + 2) + A * mi(1:plot_sample:last_index, 2), 'b', 'HandleVisibility','off', 'LineWidth', 2)

% Red represents Agent 2
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(1) + 3) + A * mi(1:plot_sample:last_index, 3), 'r', 'LineWidth', 2)
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(1) + 4) + A * mi(1:plot_sample:last_index, 4), 'r', 'HandleVisibility','off', 'LineWidth', 2)

% Yellow represents Agent 3
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(1) + 5) + A * mi(1:plot_sample:last_index, 5), 'Color', [0.9290 0.6940 0.1250], 'LineWidth', 2)
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(1) + 6) + A * mi(1:plot_sample:last_index, 6), 'Color', [0.9290 0.6940 0.1250], 'HandleVisibility','off', 'LineWidth', 2)

% Magenta represents Agent 4
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(1) + 7) + A * mi(1:plot_sample:last_index, 7), 'm', 'LineWidth', 2)
plot(t(1:plot_sample:last_index), x(1:plot_sample:last_index, i(1) + 8) + A * mi(1:plot_sample:last_index, 8), 'm', 'HandleVisibility','off', 'LineWidth', 2)

% Plot the Nash equilibrium
plot([t(1) , t(last_index)], [NE(1, 1) NE(1, 1)], '--b', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(2, 1) NE(2, 1)], '--b', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(1, 2) NE(1, 2)], '--r', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(2, 2) NE(2, 2)], '--r', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(1, 3) NE(1, 3)], 'LineStyle', '--', 'Color', [0.9290 0.6940 0.1250], 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(2, 3) NE(2, 3)], 'LineStyle', '--', 'Color', [0.9290 0.6940 0.1250], 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(1, 4) NE(1, 4)], '--m', 'HandleVisibility','off')
plot([t(1) , t(last_index)], [NE(2, 4) NE(2, 4)], '--m', 'HandleVisibility','off')
xlabel('$t$')
ylabel('$x$')
legend('Agent 1', 'Agent 2', 'Agent 3', 'Agent 4')
%title('Phase plane')
grid on

%% x states phase with oscilations
figure % position
figure_standard
hold on
%clf

% Plot the positions of the agents in the x1-x2 plane.
% Blue represents Agent 1
plot(x(1:t_delta:end, 1) + A * mi(1:t_delta:end, 1), x(1:t_delta:end, 2) + A * mi(1:t_delta:end, 2), 'b')

% Red represents Agent 2
plot(x(1:t_delta:end, 3) + A * mi(1:t_delta:end, 3), x(1:t_delta:end, 4) + A * mi(1:t_delta:end, 4), 'r')

% Yellow represents Agent 3
plot(x(1:t_delta:end, 5) + A * mi(1:t_delta:end, 5), x(1:t_delta:end, 6) + A * mi(1:t_delta:end, 6), 'Color', [0.9290 0.6940 0.1250])

% Magenta represents Agent 4
plot(x(1:t_delta:end, 7) + A * mi(1:t_delta:end, 7), x(1:t_delta:end, 8) + A * mi(1:t_delta:end, 8), 'm')

% Plot the signal source points with blue circles
plot(R_s(1, :), R_s(2, :), 'o', 'HandleVisibility','off', 'Color', 'b', 'MarkerSize', 10)

% Plot the Nash equilibrium with blue crosses
plot(NE(1, :), NE(2, :), 'x', 'HandleVisibility','off', 'Color', 'b', 'MarkerSize', 10)

% Label the axes using LaTeX interpreter
xlabel('$x_1$', 'interpreter', 'latex')
ylabel('$x_2$', 'interpreter', 'latex')
legend('Agent 1', 'Agent 2', 'Agent 3', 'Agent 4')
grid on



