function StatePlus = g(States)
    %----------------------------------------------------------------------
    % Description: Update the state after a jump
    % This function calculates the states after a jump based on current states
    %----------------------------------------------------------------------

    % Access global parameters
    global param;
    
    i       = param{2};    % Parameter indices
    T       = param{3};    % Vector of sampling periods
    T_w     = param{16};   % Vector of sampling periods for the reference controler

    % Extract the relevant states
    u       = States(i(1) + 1 : i(2));  % Control input states
    tau     = States(i(2) + 1 : i(3));  % Tau states
    tau_w   = States(i(10) + 1 : i(11)); % Tau_w states

    StatePlus = States;  % Initialize the updated state vector

    S = tau + 1e-10 >= T';          % Condition to check if tau is greater than or equal to T
    S_w = tau_w + 1e-10 >= T_w';    % Condition to check if tau_w is greater than or equal to T_w
    
    if sum(S) > 0  % If sum(S) is greater than zero, there is a state jump for tau

        % Extract parameters related to tau jump
        gamma   = param{4};
        J       = param{5};
        R       = param{6};
        m_i     = param{7};
        A       = param{9};
        beta    = param{10};

        % Extract states related to tau jump
        xi      = States(i(3) + 1 : i(4));
        mi      = States(i(4) + 1 : i(5));
        k       = States(i(5) + 1 : i(6));
        x_full  = States(i(6) + 1 : i(7));

        % Create vectors that match size of mi and u and selects the
        % states whos timmer is activated       
        S_mi = kron(S, ones(2 * m_i, 1));
        S_u = kron(S, ones(m_i, 1));

        % Update the states based on jump conditions for tau
        StatePlus(i(1) + 1 : i(2)) = u - gamma * beta * S_u .* xi;
        StatePlus(i(2) + 1 : i(3)) = ~S .* tau;
        StatePlus(i(3) + 1 : i(4)) = xi + gamma * S_u.* (2 * 1./A .* J(x_full + A .* mi(1:2:end)).* mi(1:2:end) - xi);
        StatePlus(i(4) + 1 : i(5)) = ~S_mi .* mi + S_mi .* R * mi;
        StatePlus(i(5) + 1 : i(6)) = k + S_u;
    end
    
    if sum(S_w) > 0 % If sum(S_w) is greater than zero, there is a state jump for tau_w
        
        % Extract parameters related to tau_w jump
        N  = param{8};
        c1 = param{12};
        c2 = param{13};
        c3 = param{14};
        gamma_w = param{15};
        
        % Extract states related to tau_w jump
        x       = States(i(6) + 1 : 2 : i(7));
        y       = States(i(6) + 2 : 2 : i(7));
        theta   = States(i(7) + 1 : i(8));
        t       = States(i(11) + 1 : i(12));

        theta_r = t; 
        x_ref = u(1 : 2 : end);
        y_ref = u(2 : 2 : end);
        
        wr = 1;
        xe = cos(theta(S_w)) .* (x_ref(S_w) - x(S_w)) + sin(theta(S_w)) .* (y_ref(S_w) - y(S_w));
        ye = -sin(theta(S_w)).* (x_ref(S_w) - x(S_w)) + cos(theta(S_w)) .* (y_ref(S_w) - y(S_w));
        vr = 0; 
        theta_e = theta_r - theta(S_w);

        w = wr + gamma_w * c2(S_w) * theta_e + gamma_w * ye * vr .* sinc(theta_e);
        wdot = c2(S_w) * gamma_w * (wr - w);
        xe_ = xe - c3 * w .* ye;
        v1 = vr * cos(theta_e) -c3 .* wdot .* ye + c3 * w .* (w .* xe - vr .* sin(theta_e));  
        v = v1 + c1 * xe_;

        wplus = zeros(N, 1);
        wplus(S_w) = w;
        
        vplus = zeros(N, 1);
        vplus(S_w) = v;
        
        StatePlus(i(8) + 1 : i(9)) = wplus;
        StatePlus(i(9) + 1 : i(10)) = vplus;
        StatePlus(i(10) + 1 : i(11)) = ~S_w .* tau_w;
    end

end
